@@ -188,7 +188,6 @@ static int bch2_indirect_extent_missing_error(struct btree_trans *trans,
 	u64 refd_start	= live_start	- le32_to_cpu(p.v->front_pad);
 	u64 refd_end	= live_end	+ le32_to_cpu(p.v->back_pad);
 	CLASS(printbuf, buf)();
-	int ret = 0;
 
 	BUG_ON(missing_start	< refd_start);
 	BUG_ON(missing_end	> refd_end);
@@ -205,7 +204,7 @@ static int bch2_indirect_extent_missing_error(struct btree_trans *trans,
 	prt_printf(&buf, "\nmissing reflink btree range %llu-%llu",
 		   missing_start, missing_end);
 
-	if (fsck_err(trans, reflink_p_to_missing_reflink_v, "%s", buf.buf)) {
+	if (ret_fsck_err(trans, reflink_p_to_missing_reflink_v, "%s", buf.buf)) {
 		struct bkey_i_reflink_p *new =
 			errptr_try(bch2_bkey_make_mut_noupdate_typed(trans, p.s_c, reflink_p));
 
@@ -230,7 +229,7 @@ static int bch2_indirect_extent_missing_error(struct btree_trans *trans,
 			if (missing_end < live_end)
 				new_end.offset -= live_end - missing_end;
 
-			bch2_cut_front(new_start, &new->k_i);
+			bch2_cut_front(c, new_start, &new->k_i);
 			bch2_cut_back(new_end, &new->k_i);
 
 			SET_REFLINK_P_ERROR(&new->v, true);
@@ -239,11 +238,10 @@ static int bch2_indirect_extent_missing_error(struct btree_trans *trans,
 		try(bch2_btree_insert_trans(trans, BTREE_ID_extents, &new->k_i, BTREE_TRIGGER_norun));
 
 		if (should_commit)
-			ret =   bch2_trans_commit(trans, NULL, NULL, BCH_TRANS_COMMIT_no_enospc) ?:
-				bch_err_throw(c, transaction_restart_nested);
+			try(bch2_trans_commit_lazy(trans, NULL, NULL, BCH_TRANS_COMMIT_no_enospc));
 	}
-fsck_err:
-	return ret;
+
+	return 0;
 }
 
 /*
@@ -264,30 +262,26 @@ struct bkey_s_c bch2_lookup_indirect_extent(struct btree_trans *trans,
 
 	bch2_trans_iter_init(trans, iter, BTREE_ID_reflink, POS(0, reflink_offset), iter_flags);
 	struct bkey_s_c k = bch2_btree_iter_peek_slot(iter);
-	int ret = bkey_err(k);
-	if (ret)
-		goto err;
+	if (bkey_err(k))
+		return k;
 
 	if (unlikely(!bkey_extent_is_reflink_data(k.k))) {
 		u64 missing_end = min(k.k->p.offset,
 				      REFLINK_P_IDX(p.v) + p.k->size + le32_to_cpu(p.v->back_pad));
 		BUG_ON(reflink_offset == missing_end);
 
-		ret = bch2_indirect_extent_missing_error(trans, p, reflink_offset,
-							 missing_end, should_commit);
+		int ret = bch2_indirect_extent_missing_error(trans, p, reflink_offset,
+							     missing_end, should_commit);
 		if (ret)
-			goto err;
+			return bkey_s_c_err(ret);
 	} else if (unlikely(REFLINK_P_ERROR(p.v))) {
-		ret = bch2_indirect_extent_not_missing(trans, p, should_commit);
+		int ret = bch2_indirect_extent_not_missing(trans, p, should_commit);
 		if (ret)
-			goto err;
+			return bkey_s_c_err(ret);
 	}
 
 	*offset_into_extent = reflink_offset - bkey_start_offset(k.k);
 	return k;
-err:
-	bch2_trans_iter_exit(iter);
-	return bkey_s_c_err(ret);
 }
 
 /* reflink pointer trigger */
@@ -538,11 +532,12 @@ static int bch2_make_extent_indirect(struct btree_trans *trans,
 
 static struct bkey_s_c get_next_src(struct btree_iter *iter, struct bpos end)
 {
+	struct bch_fs *c = iter->trans->c;
 	struct bkey_s_c k;
 	int ret;
 
 	for_each_btree_key_max_continue_norestart(*iter, end, 0, k, ret) {
-		if (bkey_extent_is_unwritten(k))
+		if (bkey_extent_is_unwritten(c, k))
 			continue;
 
 		if (bkey_extent_is_data(k.k))
@@ -593,8 +588,6 @@ s64 bch2_remap_range(struct bch_fs *c,
 	while ((ret == 0 ||
 		bch2_err_matches(ret, BCH_ERR_transaction_restart)) &&
 	       bkey_lt(dst_iter.pos, dst_end)) {
-		struct disk_reservation disk_res = { 0 };
-
 		bch2_trans_begin(trans);
 
 		if (fatal_signal_pending(current)) {
@@ -681,11 +674,11 @@ s64 bch2_remap_range(struct bch_fs *c,
 				min(src_k.k->p.offset - src_want.offset,
 				    dst_end.offset - dst_iter.pos.offset));
 
+		CLASS(disk_reservation, res)(c);
 		ret = bch2_extent_update(trans, dst_inum, &dst_iter,
-					 new_dst.k, &disk_res,
+					 new_dst.k, &res.r,
 					 new_i_size, i_sectors_delta,
 					 true, 0);
-		bch2_disk_reservation_put(c, &disk_res);
 	}
 
 	BUG_ON(!ret && !bkey_eq(dst_iter.pos, dst_end));
@@ -728,7 +721,6 @@ static int bch2_gc_write_reflink_key(struct btree_trans *trans,
 	const __le64 *refcount = bkey_refcount_c(k);
 	CLASS(printbuf, buf)();
 	struct reflink_gc *r;
-	int ret = 0;
 
 	if (!refcount)
 		return 0;
@@ -744,7 +736,7 @@ static int bch2_gc_write_reflink_key(struct btree_trans *trans,
 		return -EINVAL;
 	}
 
-	if (fsck_err_on(r->refcount != le64_to_cpu(*refcount),
+	if (ret_fsck_err_on(r->refcount != le64_to_cpu(*refcount),
 			trans, reflink_v_refcount_wrong,
 			"reflink key has wrong refcount:\n"
 			"%s\n"
@@ -757,10 +749,10 @@ static int bch2_gc_write_reflink_key(struct btree_trans *trans,
 			new->k.type = KEY_TYPE_deleted;
 		else
 			*bkey_refcount(bkey_i_to_s(new)) = cpu_to_le64(r->refcount);
-		ret = bch2_trans_update(trans, iter, new, 0);
+		try(bch2_trans_update(trans, iter, new, 0));
 	}
-fsck_err:
-	return ret;
+
+	return 0;
 }
 
 int bch2_gc_reflink_done(struct bch_fs *c)
